﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml.Serialization;
using System.Xml.Linq;
using System.IO; 
using System.Text;

namespace WebService1
{

    /// ////////
    //[XmlRoot(Namespace = "customer", IsNullable = false)]
    public struct Customer 
    {
        public Customer(string fname, string sname, string pnum, string saddr,
                         string city, string state, string zip, string prodOrd, string custID)
        {
            this.FirstName = fname;
            this.LastName = sname;
            this.PhoneNum = pnum;
            this.StreetAddress = saddr;
            this.City = city;
            this.State = state;
            this.ZIP = zip;
            this.ProductsOrdered = prodOrd;
            this.ID = custID;

        }

        public Customer(string name, string pnum, string saddr,
                         string city, string state, string zip, string prodOrd, string custID)
        {
            this.FirstName = name.Substring(name.IndexOf(' ')+1);
            this.LastName = name.Substring(0, name.IndexOf(','));
            this.PhoneNum = pnum;
            this.StreetAddress = saddr;
            this.City = city;
            this.State = state;
            this.ZIP = zip;
            this.ProductsOrdered = prodOrd;
            this.ID = custID;
       
        }

        public string FirstName;
        public string LastName;
        public string PhoneNum;
        public string StreetAddress;
        public string City;
        public string State;
        public string ZIP;
        public string ProductsOrdered;
        public string ID;  // Primary key
    }
    
    public struct ProdOrder
    {
        public ProdOrder(string orderid, string custID, string oType, string cost)
        {
            this.OrderNum = orderid;
            this.CustomerID = custID;
            this.OrderType = oType;
            this.Cost = cost;
            
        }

        public string OrderNum;  // Primary key
        public string CustomerID;
        public string OrderType;
        public string Cost;
    }

    public struct OrderType
    {
        public OrderType(string code, string desc)
        {
            this.Code = code;
            this.Desc = desc;
        }

        public string Code;  // Primary key
        public string Desc;
    }

    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    // [System.Web.Script.Services.ScriptService]
    public class Service1 : System.Web.Services.WebService
    {

        //***************
        //Customer Entity
        //***************
        [WebMethod]
        public Customer GetCustomer(string ID)
        //Specific Finder, needed by SPD
        {
            return getCustomer(ID);
        }
        
        [WebMethod]
        //Default Finder, used by Access
        public Customer[] GetCustomers()
        {
            return generateCustomers();
        }

        //***************
        //Order Entity
        //***************
        [WebMethod]
        public ProdOrder GetOrder(string OrderNum)
        //Specific Finder, need by SPD
        {
            return getOrder(OrderNum);
        }

        [WebMethod]
        //Default Finder, used by Access
        public ProdOrder[] GetOrders(string Location)
        {
            return generateProdOrders();
        }
        
        //***************
        //Order Type Entity
        //***************
        [WebMethod]
        //Specific finder, used by SPD
        public OrderType GetOrderType(string Code)
        {
            return getOrderType(Code);
        }

        [WebMethod]
        //Default Finder, used by Access
        public OrderType[] GetOrderTypes()
        {
            return generateOrderTypes();
        }
        
        [WebMethod]
        //BCS Scalar stereotype
        public string AcceptOrder(string OrderID)
        {
            return "Success";
        }
        
        //////////////////  Helper functions
        private String UTF8ByteArrayToString(Byte[] characters)
        {
            UTF8Encoding encoding = new UTF8Encoding();
            String constructedString = encoding.GetString(characters);
            return (constructedString);
        }

        private Byte[] StringToUTF8ByteArray(String pXmlString)
        {
            UTF8Encoding encoding = new UTF8Encoding();
            Byte[] byteArray = encoding.GetBytes(pXmlString);
            return byteArray;
        }

        public String SerializeObject(Object pObject)
        {
            try
            {
                String XmlizedString = null;
                MemoryStream memoryStream = new MemoryStream();
                XmlSerializer xs = new XmlSerializer(typeof(Customer));
                System.Xml.XmlTextWriter xmlTextWriter = new System.Xml.XmlTextWriter(memoryStream, Encoding.UTF8);
                xs.Serialize(xmlTextWriter, pObject);
                memoryStream = (MemoryStream)xmlTextWriter.BaseStream;
                XmlizedString = UTF8ByteArrayToString(memoryStream.ToArray());
                System.Console.Write(XmlizedString);
                return XmlizedString;
            }
            catch (Exception e)
            {
                System.Console.WriteLine(e);
                return null;
            }
        }

        public Object DeserializeObject ( String pXmlizedString )
        {
            XmlSerializer xs = new XmlSerializer ( typeof ( Customer ) );
            MemoryStream memoryStream = new MemoryStream ( StringToUTF8ByteArray ( pXmlizedString ) );
            System.Xml.XmlTextWriter xmlTextWriter = new System.Xml.XmlTextWriter(memoryStream, Encoding.UTF8);
     
            return xs.Deserialize ( memoryStream );
        } 

        
///////////////////
        public OrderType getOrderType (string ID)
        {
            return new OrderType("FlowersF1", "Bouqet of 12 Roses");
        }

        public OrderType[] generateOrderTypes()
        {
            OrderType[] OrderTypes = new OrderType[4];

            OrderTypes[0] = new OrderType("FlowersF1", "Bouqet of 12 Roses");
            OrderTypes[1] = new OrderType("FlowersF2", "Bouqet of 12 Roses in vase");
            OrderTypes[2] = new OrderType("ChocolateC2", "Box of chocolates");
            OrderTypes[3] = new OrderType("ChocolateC1", "Hollow Chocolate Animals");
            return OrderTypes;
        }

        public ProdOrder getOrder(string ID)
        {
            return new ProdOrder("5A23E",
                                 "0",
                                 "FlowersF1",
                                 "$4.50");
        }
        public ProdOrder[] generateProdOrders()
        {
            ProdOrder[] orders = new ProdOrder[29];
            
            orders[0] = new ProdOrder("5A23E",
                                      "0",
                                      "FlowersF1",
                                      "$4.50");
            
            orders[1] = new ProdOrder("6B45F",
                                      "1",
                                      "ChocolateC2",
                                      "$10.00");
                        
            orders[2] = new ProdOrder("7C33C",
                                      "2",
                                      "FlowersF2",
                                      "$75.00");
            
            orders[3] = new ProdOrder("8E68A",
                                      "3",
                                      "ChocolateC2",
                                      "$15.00");
            
            orders[4] = new ProdOrder("2D32E",
                                      "4",
                                      "ChocolateC1",
                                      "$7.00");
            
            orders[5] = new ProdOrder("5A24E",
                                      "5",
                                      "FlowersF1",
                                      "$4.50");

            orders[6] = new ProdOrder("6B46F",
                                      "6",
                                      "ChocolateC2",
                                      "$10.00");
            
            orders[7] = new ProdOrder("7C34C",
                                      "7",
                                      "FlowersF2",
                                      "$75.00");

            orders[8] = new ProdOrder("8E69A",
                                      "8",
                                      "ChocolateC2",
                                      "$15.00");

            orders[9] = new ProdOrder("2D33E",
                                      "9",
                                      "ChocolateC1",
                                      "$7.00");

            orders[10] = new ProdOrder("5A2EE",
                                       "10",
                                       "FlowersF1",
                                       "$4.50");

            orders[11] = new ProdOrder("6B4FF",
                                       "11",
                                       "ChocolateC2",
                                       "$10.00");
            
            orders[12] = new ProdOrder("7C3AC",
                                       "12",
                                       "FlowersF2",
                                       "$75.00");

            orders[13] = new ProdOrder("8E6BA",
                                       "13",
                                       "ChocolateC2",
                                       "$15.00");

            orders[14] = new ProdOrder("2D3AE",
                                       "14",
                                       "ChocolateC1",
                                       "$7.00");

            orders[15] = new ProdOrder("6B45A",
                                       "15",
                                       "ChocolateC2",
                                       "$10.00");
            
            orders[16] = new ProdOrder("7C33G",
                                       "16",
                                       "FlowersF2",
                                       "$75.00");

            orders[17] = new ProdOrder("8E68C",
                                       "17",
                                       "ChocolateC2",
                                       "$15.00");

            orders[18] = new ProdOrder("2D32A",
                                       "18",
                                       "ChocolateC1",
                                       "$7.00");
            
            orders[19] = new ProdOrder("5A24B",
                                       "19",
                                       "FlowersF1",
                                       "$4.50");

            orders[20] = new ProdOrder("6B46B",
                                       "20",
                                       "ChocolateC2",
                                       "$10.00");
            
            orders[21] = new ProdOrder("7C34B",
                                       "21",
                                       "FlowersF2",
                                       "$75.00");

            orders[22] = new ProdOrder("8E69D",
                                       "22",
                                       "ChocolateC2",
                                       "$15.00");

            orders[23] = new ProdOrder("2D33A",
                                       "23",
                                       "ChocolateC1",
                                       "$7.00");

            orders[24] = new ProdOrder("5A2EA",
                                       "24",
                                       "FlowersF1",
                                       "$4.50");

            orders[25] = new ProdOrder("6B4FA",
                                       "25",
                                       "ChocolateC2",
                                       "$10.00");
            
            orders[26] = new ProdOrder("7C3AB",
                                       "26",
                                       "FlowersF2",
                                       "$75.00");

            orders[27] = new ProdOrder("8E6BF",
                                       "0",
                                       "ChocolateC2",
                                       "$15.00");

            orders[28] = new ProdOrder("2D3AA",
                                       "1",
                                       "ChocolateC1",
                                       "$7.00");
            return orders;
        }
///////////////////
        public Customer getCustomer(string ID)
        {
            return new Customer("Conrad, Jeff",
                                "789-555-0191",
                                "4567 1st Avenue",
                                "Redmond",
                                "WA",
                                "98052",
                                "Access", "1");
        }

        public Customer[] generateCustomers()
        {
            Customer[] customers = new Customer[27];

            customers[0] = new Customer("Conrad, Jeff",
                                        "789-555-0191",
                                        "4567 1st Avenue",
                                        "Redmond",
                                        "WA",
                                        "98052",
                                        "Access", "1");

            customers[1] = new Customer("Sousa, Anibal",
                                        "234-555-0162",
                                        "4567 Main Street",
                                        "Grosse Pointe Shores",
                                        "MI",
                                        "48236",
                                        "Access", "2");

            customers[2] = new Customer("Riegle, Jennifer",
                                        "123-555-0181",
                                        "2345 2nd Avenue",
                                        "Wallaceton",
                                        "PA",
                                        "16876",
                                        "Access", "3");

            customers[3] = new Customer("Jensen, Kathleen Gail",
                                        "123-555-0152",
                                        "1234 4th Boulevard",
                                        "Warriors Mark",
                                        "PA",
                                        "16877",
                                        "Access", "4");

            customers[4] = new Customer("Francis, Cat",
                                        "345-555-0187",
                                        "4567 Main Avenue",
                                        "Lashmeet",
                                        "WV",
                                        "24733",
                                        "Access", "5");

            customers[5] = new Customer("Yong, Joe",
                                        "123-555-0185",
                                        "5678 Main Boulevard",
                                        "Shawville",
                                        "PA",
                                        "16873",
                                        "Access", "6");

            customers[6] = new Customer("Linggoputro, Harry",
                                        "567-555-0171",
                                        "7890 4th Boulevard",
                                        "Yorktown",
                                        "VA",
                                        "23691",
                                        "Access", "7");

            customers[7] = new Customer("Gibson, Mary",
                                        "345-555-0183",
                                        "5678 Central Drive",
                                        "Kellysville",
                                        "WV",
                                        "24732",
                                        "Access", "8");

            customers[8] = new Customer("Smith, Aaron",
                                        "789-555-0166",
                                        "4567 Main Avenue",
                                        "Fresno",
                                        "CA",
                                        "93707",
                                        "Access", "9");

            customers[9] = new Customer("Michaels, Tom",
                                        "456-555-0153",
                                        "7890 Lincoln Boulevard",
                                        "Kennerdell",
                                        "PA",
                                        "16374",
                                        "Access", "10");

            customers[10] = new Customer("Dixon, Andrew",
                                         "456-555-0174",
                                         "2345 3rd Street",
                                         "Kalamazoo",
                                         "MI",
                                         "49008",
                                         "Access", "11");

            customers[11] = new Customer("Stoklasa, Jan",
                                         "456-555-0114",
                                         "7890 Lincoln Avenue",
                                         "Youngsville",
                                         "PA",
                                         "16371",
                                         "Access", "12");

            customers[12] = new Customer("West, Paul",
                                         "456-555-0177",
                                         "3456 Main Boulevard",
                                         "Kalamazoo",
                                         "MI",
                                         "49004",
                                         "Access", "13");
            
            customers[13] = new Customer("Kresnadi, Mario",
                                         "234-555-0169",
                                         "1234 3rd Parkway",
                                         "Grosse Pointe Farms",
                                         "MI",
                                         "48236",
                                         "Access", "14");

            customers[14] = new Customer("Peterson, Palle",
                                         "456-555-0170",
                                         "1234 Central Parkway",
                                         "Emlenton",
                                         "PA",
                                         "16373",
                                         "Access", "15");

            customers[15] = new Customer("Matthews, Joseph",
                                         "234-555-0173",
                                         "5678 2nd Avenue",
                                         "Detroit",
                                         "MI",
                                         "48231",
                                         "Access", "16");

            customers[16] = new Customer("Viescas, John",
                                         "789-555-0152",
                                         "379 Amherst St PMB 215",
                                         "Nashua",
                                         "NH",
                                         "03063",
                                         "Access", "17");

            customers[17] = new Customer("Curran, Douglas",
                                         "789-555-0150",
                                         "2345 Willow Drive",
                                         "Easton",
                                         "CA",
                                         "93706",
                                         "Access", "18");

            customers[18] = new Customer("Yang, Shengda",
                                         "789-555-0152",
                                         "4567 Central Street",
                                         "Fresno",
                                         "CA",
                                         "93710",
                                         "Access", "19");

            customers[19] = new Customer("Koczka, Daniel",
                                         "678-555-0155",
                                         "4567 Central Street",
                                         "Sandgap",
                                         "KY",
                                         "40481",
                                         "Access", "20");

            customers[20] = new Customer("Lannin, Adrian",
                                         "345-555-0194",
                                         "4567 4th Boulevard",
                                         "Kingston",
                                         "IL",
                                         "60145",
                                         "Access", "21");

            customers[21] = new Customer("Wroblewska, Magdalena",
                                         "456-555-0182",
                                         "5678 Willow Avenue",
                                         "Bernardston",
                                         "MA",
                                         "01337",
                                         "Access", "22");

            customers[22] = new Customer("Martin, Mindy",
                                         "234-555-0152",
                                         "4567 2nd Parkway",
                                         "Grandy",
                                         "NC",
                                         "27939",
                                         "Access", "23");

            customers[23] = new Customer("Ready, Rodrigo",
                                         "678-555-0165",
                                         "2345 Church Avenue",
                                         "Stanford",
                                         "KY",
                                         "40484",
                                         "Access", "24");

            customers[24] = new Customer("Sandberg, Mikael",
                                         "456-555-0174",
                                         "4567 3rd Parkway",
                                         "Colrain",
                                         "MA",
                                         "01340",
                                         "Access", "25");

            customers[25] = new Customer("Zabokritski, Eugene",
                                         "456-555-0185",
                                         "7890 3rd Parkway",
                                         "Wahpeton",
                                         "ND",
                                         "58074",
                                         "Access", "26");

            customers[26] = new Customer("Zulechner, Markus",
                                         "234-555-0191",
                                         "1234 Lincoln Boulevard",
                                         "Harrellsville",
                                         "NC",
                                         "27942",
                                         "Access", "27");         
            return customers;
        }

    }

}
